package com.example.akka.test.actors;


import com.example.akka.test.msg.IdsRequired;

import akka.actor.ActorRef;
import akka.actor.Props;
import akka.actor.UntypedActor;
import akka.japi.Creator;

public class CreateContent extends UntypedActor {

	private ActorRef unusedIDsActor;
	private ActorRef recordsActor;
	private int fullMsgCount = 1500000;

	public CreateContent() {
		unusedIDsActor = getContext().actorOf(Props.create(UnusedIds.class),
				"unusedIds");
		recordsActor = getContext().actorOf(Props.create(CreateRecords.class),
				"records");
	}

	public static Props props() {
		return Props.create(new Creator<CreateContent>() {
			private static final long serialVersionUID = 1L;

			public CreateContent create() throws Exception {
				return new CreateContent();
			}
		});
	}
	
	private void orchestrate() {
		// We use 50% more Ids than have been initialised 
		// UnusedIds will generate more for us
		IdsRequired idsReq = new IdsRequired();
		for (int slotCount = 0; slotCount < 96; slotCount++) {
			if((slotCount >= 0 && slotCount < 12) ||
					(slotCount >= 84 && slotCount < 96)) {
				// Distribute 5% for each of these 12 slots
				idsReq.setNumReq((int)(fullMsgCount/12*0.05+0.5));
			} else if((slotCount >= 12 && slotCount < 24) ||
					(slotCount >= 72 && slotCount < 84)) {
				// Distribute 10% for each of these 12 slots
				idsReq.setNumReq((int)(fullMsgCount/12*0.10+0.5));
			} else if((slotCount >= 24 && slotCount < 36) ||
					(slotCount >= 60 && slotCount < 72)) {
				// Distribute 15% for each of these 12 slots
				idsReq.setNumReq((int)(fullMsgCount/12*0.15+0.5));
			} else if(slotCount >= 36 && slotCount < 60) {
				// Distribute 20% for these 24 slots
				idsReq.setNumReq((int)(fullMsgCount/12*0.20+0.5));
			}
			idsReq.setSlot(slotCount);
			recordsActor.tell(idsReq, getSelf());
		}
		this.getContext().system().shutdown();
	}
	
	@Override
	public void onReceive(Object msg) throws Exception {
		if( msg instanceof String) {
			String runState = ((String) msg);

			if(runState.equals("Start")) {
				unusedIDsActor.tell(new Long(1000000l), getSelf());
			} else if (runState.equals("UIdsGenerated")) {
				orchestrate();
			}
		}
	}
}
