package com.example.akka.test.actors;

import java.util.ArrayList;

import com.example.akka.test.msg.IdsRequired;
import com.example.akka.test.msg.UsedId.State;

import akka.actor.ActorRef;
import akka.actor.Props;
import akka.actor.Terminated;
import akka.actor.UntypedActor;
import akka.japi.Creator;

public class CreateRecords extends UntypedActor {

	private ActorRef liveIdsActor;
	private ActorRef outputActor;

	public CreateRecords() {
		liveIdsActor = getContext().actorOf(Props.create(LiveIds.class),
				"liveIds");
		this.getContext().watch(liveIdsActor);
		outputActor = getContext().actorOf(Props.create(Output.class),
				"outputIds");
	}

	public static Props props() {
		return Props.create(new Creator<CreateRecords>() {
			private static final long serialVersionUID = 1L;

			public CreateRecords create() throws Exception {
				return new CreateRecords();
			}
		});
	}

	private void buildRecord(IdsRequired idsReq) {
		ArrayList<String> records = new ArrayList<String>();

		for (int i = 0; i < idsReq.getIds().size(); i++) {
			StringBuffer sb = new StringBuffer(60);
			sb.append("A Test record with Id - ");
			sb.append(idsReq.getIds().get(i));
			sb.append(" at Slot - ");
			sb.append(idsReq.getSlot());
			sb.append(" at Stage - ");
			sb.append(idsReq.getStage());
			records.add(sb.toString());
		}
		idsReq.setIds(records);
		outputActor.tell(idsReq, getSelf());
	}

	@Override
	public void onReceive(Object msg) throws Exception {
		if (msg instanceof IdsRequired) {
			IdsRequired idsReq = (IdsRequired) msg;
			// Checks to see if the ArrayList is empty,
			// if it is then it get Ids
			if (((IdsRequired) msg).getIds() == null) {
				liveIdsActor.tell(idsReq, getSelf());
			} else {
				int count = (int) (idsReq.getNumReq()/5+0.5);
				idsReq.setStage(State.STAGE1);
				idsReq.setNumReq(count);
				buildRecord(idsReq);
				idsReq.setStage(State.STAGE2);
				idsReq.setNumReq(count);
				buildRecord(idsReq);
				idsReq.setStage(State.STAGE3);
				idsReq.setNumReq(count);
				buildRecord(idsReq);
				idsReq.setStage(State.STAGE4);
				idsReq.setNumReq(count);
				buildRecord(idsReq);
				idsReq.setStage(State.STAGE5);
				idsReq.setNumReq(count);
				buildRecord(idsReq);
			}
		} else {
			unhandled(msg);
		}
	}
}
