package com.example.akka.test.actors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import scala.concurrent.Await;
import scala.concurrent.Future;
import scala.concurrent.duration.Duration;

import com.example.akka.test.Main;
import com.example.akka.test.msg.IdsRequired;
import com.example.akka.test.msg.UsedId;
import com.example.akka.test.msg.UsedId.State;

import akka.actor.ActorSelection;
import akka.actor.Props;
import akka.actor.UntypedActor;
import akka.japi.Creator;
import akka.pattern.Patterns;
import akka.util.Timeout;

public class LiveIds extends UntypedActor {

	HashMap<Integer, List<String>> S1Ids = new HashMap<Integer, List<String>>();
	HashMap<Integer, List<String>> S2Ids = new HashMap<Integer, List<String>>();
	HashMap<Integer, List<String>> S3Ids = new HashMap<Integer, List<String>>();
	HashMap<Integer, List<String>> S4Ids = new HashMap<Integer, List<String>>();
	HashMap<Integer, List<String>> S5Ids = new HashMap<Integer, List<String>>();

	private int Stg1Lookback = 8;
	private int Stg2Lookback = 16;
	private int Stg3Lookback = 32;
	private int Stg4Lookback = 16;
	private int Stg5Lookback = 24;
	
	private Timeout timeout = new Timeout(Duration.create(30, "seconds"));


	public LiveIds() {
		
	}

	public static Props props() {
		return Props.create(new Creator<LiveIds>() {
			private static final long serialVersionUID = 1L;

			public LiveIds create() throws Exception {
				return new LiveIds();
			}
		});
	}
	
	private void addToStore(UsedId msg) {
		if (S1Ids.get(msg.getStage().ordinal()) == null) {
			S1Ids.put(msg.getStage().ordinal(), msg.getIds());
		} else {
			S1Ids.get(msg.getStage().ordinal()).addAll(msg.getIds());
		}
	}
	
	private IdsRequired getIds(IdsRequired msg) {
		UsedId usedSet = new UsedId();
		usedSet.setStage(msg.getStage());
		ArrayList<String> ids = null;
		if (msg.getStage() == State.STAGE1) {
			ids = populateUIds(msg, S1Ids, Stg1Lookback);
		} else if (msg.getStage() == State.STAGE2) {
			ids = populateUIds(msg, S2Ids, Stg2Lookback);
		} else if (msg.getStage() == State.STAGE3) {
			ids = populateUIds(msg, S3Ids, Stg3Lookback);
		} else if (msg.getStage() == State.STAGE4) {
			ids = populateUIds(msg, S4Ids, Stg4Lookback);
		} else if (msg.getStage() == State.STAGE5) {
			ids = populateUIds(msg, S5Ids, Stg5Lookback);
		}
		msg.setIds(ids);
		usedSet.setIds(ids);
		addToStore(usedSet);
		return msg;
	}
	
	private ArrayList<String> populateUIds(IdsRequired msg,
			HashMap<Integer, List<String>> sIds, int stgLookback) {
		int numReq = msg.getNumReq();
		ArrayList<String> availUIds = new ArrayList<String>(0);

		int i = 0;
		while (availUIds.size() < numReq) {
			// Look one slot closer, after i increments > 0
			int lookbackSlot = msg.getSlot() - stgLookback + i;
			if (lookbackSlot >= 0 && sIds.get(lookbackSlot) != null) {
				List<String> availBack = new ArrayList<String>(sIds.get(lookbackSlot));
				if (availBack != null && availBack.size() >= numReq) {
					// Take all from here
					availUIds.addAll(availBack.subList(0, numReq));
					List<String> remainingBack = availBack.subList(numReq, availBack.size());
					sIds.put(msg.getSlot(), remainingBack);
					numReq = 0;
					break;
				} else if(availBack != null){
					// Take what we can from here and continue to search each
					// side
					availUIds.addAll(availBack);
					numReq -= availBack.size();
					sIds.put(msg.getSlot(), new ArrayList<String>());
				}
			}
			lookbackSlot = msg.getSlot() - stgLookback - i;
			// Look one slot further back
			if (i > 0 && lookbackSlot > 0 && sIds.get(lookbackSlot) != null) {
				List<String> avail = new ArrayList<String>(sIds.get(lookbackSlot));
				if (avail != null && avail.size() >= numReq) {
					// Take all from here
					availUIds.addAll(avail.subList(0, numReq));
					List<String> remainingFwd = avail.subList(numReq, avail.size());
					sIds.put(msg.getSlot(), remainingFwd);
					numReq = 0;
					break;
				} else if (avail != null){
					// Take what we can from here and continue to search
					// each side
					availUIds.addAll(avail);
					numReq -= avail.size();
					sIds.put(msg.getSlot(), new ArrayList<String>());
				}
			}
			i++;
			if (i >= stgLookback / 4) {
				break;
			}
		}
		if (numReq > 0) {
			ActorSelection unusedIdsActor = Main.actorSystem.actorSelection("/user/contentActor/unusedIds");
			Future<Object> future = Patterns.ask(unusedIdsActor,
					new Integer(numReq), timeout);
			try {
				@SuppressWarnings("unchecked")
				List<String> availUnUsed = (ArrayList<String>) Await.result(future,
						timeout.duration());
				for(String availId : availUnUsed) {
					availUIds.add(availId);
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return availUIds;
	}

	@Override
	public void onReceive(Object msg) throws Exception {
		if (msg instanceof UsedId) {
			addToStore((UsedId) msg);
		} else if (msg instanceof IdsRequired) {
			IdsRequired ids = getIds((IdsRequired) msg);
			getSender().tell(ids, getSelf());
		}
	}
}
